/*	scan.c

	AFAPI scan (parallel prefix) function definitions for
	PAPERS_JR.  Like reductions, for NPROC=2, these all become
	very simple.  In this case, we just have PE0 broadcast to PE1.

	July 1996 by H. Dietz
*/

#include "afapi.h"
#include "inline.h"

/*	p_scanAnd() functions...
*/

#define	P_DECLARE(suffix) \
p_##suffix \
p_scanAnd##suffix(register p_##suffix datum) \
{ \
	if (IPROC == 0) { \
		p_bcastPut##suffix(datum); \
		return(datum); \
	} else { \
		return(datum & p_bcastGet##suffix()); \
	} \
}

P_DECLARE(1u)
P_DECLARE(8)
P_DECLARE(8u)
P_DECLARE(16)
P_DECLARE(16u)
P_DECLARE(32)
P_DECLARE(32u)
P_DECLARE(64)
P_DECLARE(64u)
P_DECLARE(bar)

#undef	P_DECLARE


p_ptr
p_scanAndptr(register p_ptr datum)
{
	return((p_ptr) p_scanAnd32u((p_32u) datum));
}

/*	p_scanOr() functions...
*/

#define	P_DECLARE(suffix) \
p_##suffix \
p_scanOr##suffix(register p_##suffix datum) \
{ \
	if (IPROC == 0) { \
		p_bcastPut##suffix(datum); \
		return(datum); \
	} else { \
		return(datum | p_bcastGet##suffix()); \
	} \
}

P_DECLARE(1u)
P_DECLARE(8)
P_DECLARE(8u)
P_DECLARE(16)
P_DECLARE(16u)
P_DECLARE(32)
P_DECLARE(32u)
P_DECLARE(64)
P_DECLARE(64u)
P_DECLARE(bar)

#undef	P_DECLARE


p_ptr
p_scanOrptr(register p_ptr datum)
{
	return((p_ptr) p_scanOr32u((p_32u) datum));
}

/*	p_scanXor() functions...

	Implementation does NPROC-1 broadcasts.
*/

#define	P_DECLARE(suffix) \
p_##suffix \
p_scanXor##suffix(register p_##suffix datum) \
{ \
	if (IPROC == 0) { \
		p_bcastPut##suffix(datum); \
		return(datum); \
	} else { \
		return(datum ^ p_bcastGet##suffix()); \
	} \
}

P_DECLARE(1u)
P_DECLARE(8)
P_DECLARE(8u)
P_DECLARE(16)
P_DECLARE(16u)
P_DECLARE(32)
P_DECLARE(32u)
P_DECLARE(64)
P_DECLARE(64u)
P_DECLARE(bar)

#undef	P_DECLARE


p_ptr
p_scanXorptr(register p_ptr datum)
{
	return((p_ptr) p_scanXor32u((p_32u) datum));
}

/*	p_scanMin() functions...
*/

#define	P_DECLARE(suffix) \
p_##suffix \
p_scanMin##suffix(register p_##suffix datum) \
{ \
	if (IPROC == 0) { \
		p_bcastPut##suffix(datum); \
		return(datum); \
	} else { \
		register p_##suffix t = p_bcastGet##suffix(); \
		return((datum < t) ? datum : t); \
	} \
}

P_ALPHA

#undef	P_DECLARE

/*	p_scanMax() functions...
*/

#define	P_DECLARE(suffix) \
p_##suffix \
p_scanMax##suffix(register p_##suffix datum) \
{ \
	if (IPROC == 0) { \
		p_bcastPut##suffix(datum); \
		return(datum); \
	} else { \
		register p_##suffix t = p_bcastGet##suffix(); \
		return((datum > t) ? datum : t); \
	} \
}

P_ALPHA

#undef	P_DECLARE

/*	p_scanAdd() functions...
*/

#define	P_DECLARE(suffix) \
p_##suffix \
p_scanAdd##suffix(register p_##suffix datum) \
{ \
	if (IPROC == 0) { \
		p_bcastPut##suffix(datum); \
		return(datum); \
	} else { \
		return(datum + p_bcastGet##suffix()); \
	} \
}

P_GAMMA

#undef P_DECLARE

/*	p_scanMul() functions...
*/

#define	P_DECLARE(suffix) \
p_##suffix \
p_scanMul##suffix(register p_##suffix datum) \
{ \
	if (IPROC == 0) { \
		p_bcastPut##suffix(datum); \
		return(datum); \
	} else { \
		return(datum * p_bcastGet##suffix()); \
	} \
}

P_GAMMA

#undef P_DECLARE
